C  Conversions to and from the geo (geographic) and the map
C  (Map-oriented)  coordinate systems.
C
C  The geo system is a 3-D Cartesian coordinate system, with origin
C  at the Center of the Earth, with the x_3 axis pointing to the North
C  Pole, the x_1 axis pointing to the Equator at the Greenwich Meridian,
C  and the x_2 axis by the right hand rule pointing to the Equator
C  at 90 E.
C
C  In the map system, the axis of the map projection passes through the
C  center of the Earth, and through a point on the surface that acts as the
c  "Pole" of the map (Which will coincide with the North or South Pole,
C  unless the projection is Oblique or Transverse).  The x_3 axis of the
C  map coordinate system is aligned with this map pole.  In Lambert and
C  Mercator projections, a "Cut" extends from the map pole along a great
C  circle to its antipode; the unrolled map is discontinuous there.  The
C  x_1 axis of the map coordinate system is diametrically opposite this
C  cut, and 90 degrees from the pole; the x_2 axis is selected to complete
C  a right hand rule.
C
C  The coefficients of the map coordinate system relative to the geo system
C  are given in elements 2 through 10 of the stcprm array.
C
      subroutine basg2m(stcprm, xin, xout)
C  receives the vector xin, giving the components of a point in the geo
C  system; returns xout, the components of the same point in the map system
	
	include 'cmapf.fi'
      real stcprm(k_mparam)

      dimension xin(3),xout(3)
      ifind(l,k) = k_rot - 4 + 3*l + k
        do k=1,3
          xout(k) = 0.
        enddo
        do l=1,3
          do k=1,3
            xout(l) = xout(l) + xin(k) * stcprm(ifind(l,k))
          enddo
        enddo
        return
      end
